      REAL FUNCTION cumbin(s,xn,pr)
C
C**********************************************************************
C
C     REAL FUNCTION CUMBIN(S,XN,PBIN)
C                    CUmulative BINomial distribution
C
C
C                              Function
C
C
C     Returns the probability   of 0  to  S  successes in  XN   binomial
C     trials, each of which has a probability of success, PBIN.
C
C
C                              Arguments
C
C
C     S --> The upper limit of cumulation of the binomial distribution.
C                                                  S is REAL
C
C     XN --> The number of binomial trials.
C                                                  XN is REAL
C
C     PBIN --> The probability of success in each binomial trial.
C                                                  PBIN is REAL
C
C
C                              Method
C
C
C     Formula  26.5.24    of   Abramowitz  and    Stegun,  Handbook   of
C     Mathematical   Functions (1966) is   used  to reduce the  binomial
C     distribution  to  the  cumulative    beta distribution.
C
C**********************************************************************
C     .. Scalar Arguments ..
      REAL pr,s,xn
C     ..
C     .. External Functions ..
      DOUBLE PRECISION dumbet
      EXTERNAL dumbet
C     ..
C     .. Executable Statements ..
      IF (.NOT. (s.LT.xn)) GO TO 10
      cumbin = real(1.D0-dumbet(dble(pr),dble(s)+1.D0,dble(xn)-dble(s)))
      GO TO 20

   10 cumbin = 1.0
   20 RETURN

      END
