      REAL FUNCTION cumf(f,dfn,dfd)
C
C**********************************************************************
C
C     REAL FUNCTION CUMF(F,DFN,DFD)
C                    CUMulative F distribution
C
C
C                              Function
C
C
C     Computes  the  integral from  0  to  F of  the f-density  with DFN
C     and DFD degrees of freedom.
C
C
C                              Arguments
C
C
C     F --> Upper limit of integration of the f-density.
C                                                  F is REAL
C
C     DFN --> Degrees of freedom of the numerator sum of squares.
C                                                  DFN is REAL
C
C     DFD --> Degrees of freedom of the denominator sum of squares.
C                                                  DFD is REAL
C
C
C                              Method
C
C
C     Formula  26.5.28 of  Abramowitz and   Stegun   is  used to  reduce
C     the cumulative F to a cumulative beta distribution.
C
C
C                              Note
C
C
C     If F is less than or equal to 0, 0 is returned.
C
C**********************************************************************
C     .. Scalar Arguments ..
      REAL dfd,dfn,f
C     ..
C     .. Local Scalars ..

      DOUBLE PRECISION ddfd,ddfn,df,dsum,dumf,dummy,prod,xx,yy
      INTEGER ierr
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC dble,real
C     ..
C     .. Parameters ..
      DOUBLE PRECISION half
      PARAMETER (half=0.5D0)
      DOUBLE PRECISION done
      PARAMETER (done=1.0D0)
C     ..
C     .. External Subroutines ..
      EXTERNAL bratio
C     ..
C     .. Executable Statements ..

      IF (.NOT. (f.LE.0.0)) GO TO 10
      cumf = 0.0
      RETURN

   10 ddfn = dble(dfn)
C
C     XX is such that the incomplete beta with parameters
C     DFD/2 and DFN/2 evaluated at XX is 1 - CUMF
C
C     YY is 1 - XX
C
C     Calculate the smaller of XX and YY accurately
C
      ddfd = dble(dfd)
      df = dble(f)
      prod = ddfn*df
      dsum = ddfd + prod
      xx = ddfd/dsum
      IF (xx.GT.half) THEN
          yy = prod/dsum
          xx = done - yy

      ELSE
          yy = done - xx
      END IF

      CALL bratio(ddfd*half,ddfn*half,xx,yy,dummy,dumf,ierr)
      cumf = real(dumf)
      RETURN

      END
