      REAL FUNCTION cumnbn(s,xn,pr)
C**********************************************************************
C
C     REAL FUNCTION CUMNNBN(S,XN,PR)
C                    CUmulative Negative BINomial distribution
C
C
C                              Function
C
C
C     Returns the probability that it there will be S or fewer failures
C     before there are XN successes, with each binomial trial having
C     a probability of success PR.
C
C     Prob(# failures = S | XN successes, PR)  =
C                        ( XN + S - 1 )
C                        (            ) * PR^XN * (1-PR)^S
C                        (      S     )
C
C
C                              Arguments
C
C
C     S --> The number of failures
C                                                  S is REAL
C
C     XN --> The number of successes
C                                                  XN is REAL
C
C     PR --> The probability of success in each binomial trial.
C                                                  PR is REAL
C
C
C                              Method
C
C
C     Formula  26.5.26    of   Abramowitz  and    Stegun,  Handbook   of
C     Mathematical   Functions (1966) is   used  to reduce the  negative
C     binomial distribution to the cumulative beta distribution.
C
C**********************************************************************
C     .. Scalar Arguments ..
      REAL pr,s,xn
C     ..
C     .. External Functions ..
      DOUBLE PRECISION dumbet
      EXTERNAL dumbet
C     ..
C     .. Executable Statements ..
      cumnbn = real(dumbet(dble(pr),dble(xn),dble(s)+1.D0))

      RETURN

      END
