      REAL FUNCTION cumt(t,df)
C
C**********************************************************************
C
C     REAL FUNCTION CUMT(T,DF)
C                    CUMulative T-distribution
C
C
C                              Function
C
C
C     Computes the integral from -infinity to T of the t-density.
C
C
C                              Arguments
C
C
C     T --> Upper limit of integration of the t-density.
C                                                  T is REAL
C
C     DF --> Degrees of freedom of the t-distribution.
C                                                  DF is REAL
C
C
C                              Method
C
C
C     Formula 26.5.27   of     Abramowitz  and   Stegun,    Handbook  of
C     Mathematical Functions  is   used   to  reduce the  t-distribution
C     to an incomplete beta.
C
C**********************************************************************

C     .. Scalar Arguments ..
      REAL df,t
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION xx
      REAL a
C     ..
C     .. External Functions ..
      DOUBLE PRECISION dumbet
      EXTERNAL dumbet
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC dble,real
C     ..
C     .. Executable Statements ..

      xx = dble(df)/ (dble(df)+dble(t)**2)
      a = real(1.D0-dumbet(xx,dble(df/2.0),0.5D0))
      IF (.NOT. (t.LE.0.0)) GO TO 10
      cumt = 0.5* (1.0-a)
      GO TO 20

   10 cumt = 0.5* (1.0+a)
   20 RETURN

      END
