      DOUBLE PRECISION FUNCTION dumbet(xx,xa,xb)
C**********************************************************************
C
C     DOUBLE PRECISION FUNCTION DUMBET(X,A,B)
C          Double precision cUMulative incomplete BETa distribution
C
C
C                              Function
C
C
C     Returns the cdf    to   X  of the  incomplete beta    distribution
C     with  parameters a  and b.   This   is the  integral  from  0 to x
C     of (1/B(a,b))*f(t)) where f(t) = t**(a-1) * (1-t)**(b-1)
C
C
C                              Arguments
C
C
C     X --> Upper limit of integration.
C                                        X is DOUBLE PRECISION
C
C     A --> First parameter of the beta distribution.
C                                        A is DOUBLE PRECISION
C
C     B --> Second parameter of the beta distribution.
C                                        B is DOUBLE PRECISION
C
C
C                              Method
C
C
C     Calls the routine BRATIO.
C
C                                   References
C
C     Didonato, Armido R. and Morris, Alfred H. Jr. (1992) Algorithim
C     708 Significant Digit Computation of the Incomplete Beta Function
C     Ratios. ACM ToMS, Vol.18, No. 3, Sept. 1992, 360-373.
C
C**********************************************************************
C     .. Scalar Arguments ..
      DOUBLE PRECISION xx,xa,xb
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION yy,w,w1
      INTEGER ierr
C     ..
C     .. External Routines ..
      EXTERNAL bratio
C     ..
C     .. Executable Statements ..
      IF (.NOT. (xx.LE.0.0D0)) GO TO 10
      dumbet = 0.0D0
      RETURN

   10 IF (.NOT. (xx.GE.1.0D0)) GO TO 20
      dumbet = 1.0D0
      RETURN

   20 yy = 1.0D0 - xx

C     Call bratio routine

      CALL bratio(xa,xb,xx,yy,w,w1,ierr)
      dumbet = w

      RETURN

      END
