      DOUBLE PRECISION FUNCTION dumnor(x)
C**********************************************************************
C
C     DOUBLE PRECISION FUNCTION DUMNOR(X)
C
C
C                              Function
C
C
C     Computes the cumulative  of    the  normal   distribution,   i.e.,
C     the integral from -infinity to x of
C          (1/sqrt(2*pi)) exp(-u*u/2) du
C
C
C                              Method
C
C
C     The    rational function  approximation   from pages   90  - 92 of
C     Kennedy  and Gentle,  Statistical  Computing,  Marcel  Dekker,  NY
C     1980.
C
C
C                              Arguments
C
C
C     X --> Argument at which cumulative normal is evaluated
C                    DOUBLE PRECISION X
C
C**********************************************************************
C
C
C     PIM12 IS PI**(-1/2)
C     SQRT2 IS SQRT(2)
C
C
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION x
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION derf,derfc,pim12,sqrt2,z,z2,zm2
      LOGICAL qdirct
C     ..
C     .. Local Arrays ..
      DOUBLE PRECISION xden1(4),xden2(8),xden3(5),xnum1(4),xnum2(8),
     +                 xnum3(5)
C     ..
C     .. External Functions ..
      DOUBLE PRECISION devlpl,dlanor
      EXTERNAL devlpl,dlanor
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC abs,exp
C     ..
C     .. Data statements ..
      DATA xnum1/2.4266795523053175D2,2.1979261618294152D1,
     +     6.9963834886191355D0,-3.5609843701815385D-2/
      DATA xden1/2.1505887586986120D2,9.1164905404514901D1,
     +     1.5082797630407787D1,1.0000000000000000D0/
      DATA xnum2/3.004592610201616005D2,4.519189537118729422D2,
     +     3.393208167343436870D2,1.529892850469404039D2,
     +     4.316222722205673530D1,7.211758250883093659D0,
     +     5.641955174789739711D-1,-1.368648573827167067D-7/
      DATA xden2/3.004592609569832933D2,7.909509253278980272D2,
     +     9.313540948506096211D2,6.389802644656311665D2,
     +     2.775854447439876434D2,7.700015293522947295D1,
     +     1.278272731962942351D1,1.000000000000000000D0/
      DATA xnum3/-2.99610707703542174D-3,-4.94730910623250734D-2,
     +     -2.26956593539686930D-1,-2.78661308609647788D-1,
     +     -2.23192459734184686D-2/
      DATA xden3/1.06209230528467918D-2,1.91308926107829841D-1,
     +     1.05167510706793207D0,1.98733201817135256D0,
     +     1.00000000000000000D0/
      DATA pim12/0.5641895835477562869480795D0/
      DATA sqrt2/1.4142135623730950488D0/
C     ..
C     .. Executable Statements ..
      IF (.NOT. (abs(x).LT.1.0E-30)) GO TO 10
      dumnor = 0.5
      RETURN

      GO TO 50

   10 IF (.NOT. (x.LE.-38.0)) GO TO 20
      dumnor = 0.0
      RETURN

      GO TO 50

   20 IF (.NOT. (x.LE.-15.0)) GO TO 30
      dumnor = exp(dlanor(x))
      RETURN

      GO TO 50

   30 IF (.NOT. (x.GT.6.0)) GO TO 40
      dumnor = 1.0
      RETURN

      GO TO 50

   40 CONTINUE
   50 z = abs(x/sqrt2)
      z2 = z*z
      zm2 = 1.0D0/z2
      IF (z.LT.0.5D0) THEN
          derf = z*devlpl(xnum1,4,z2)/devlpl(xden1,4,z2)
          qdirct = .TRUE.

      ELSE IF (z.LT.4.0D0) THEN
          derfc = exp(-z2)*devlpl(xnum2,8,z)/devlpl(xden2,8,z)
          qdirct = .FALSE.

      ELSE
          derfc = (exp(-z2)/z)* (pim12+zm2*devlpl(xnum3,5,zm2)/
     +            devlpl(xden3,5,zm2))
          qdirct = .FALSE.
      END IF

      IF (.NOT. (x.GE.0.0)) GO TO 60
      IF (.NOT. (qdirct)) derf = 1.0D0 - derfc
      dumnor = (1.0D0+derf)/2.0D0
      GO TO 70

   60 IF (qdirct) derfc = 1.0D0 - derf
      dumnor = derfc/2.0D0
   70 RETURN

      END
