      Real*8 FUNCTION invnor(p)
C
C**********************************************************************
C
C     Real*8 FUNCTION INVNOR(P)
C                    NORmal distribution INVerse
C
C
C                              Function
C
C
C     Returns X  such that CUMNOR(X)  =   P,  i.e., the  integral from -
C     infinity to X of (1/SQRT(2*PI)) EXP(-U*U) dU is P
C
C
C                              Arguments
C
C
C     P --> The probability whose normal deviate is sought.
C		     P is Real*8
C
C
C                              Method
C
C
C     The  rational   function   on  page 95    of Kennedy  and  Gentle,
C     Statistical Computing, Marcel Dekker, NY , 1980.
C
C
C                              Note
C
C
C     If P .lt. 1.0e-20 then INVNOR returns -10.0.
C     If P .ge. 1.0 then INVNOR returns 10.0.
C
C**********************************************************************
C
C     .. Scalar Arguments ..
      Real*8 p
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION sign,y,z
C     ..
C     .. Local Arrays ..
      DOUBLE PRECISION xden(5),xnum(5)
C     ..
C     .. External Functions ..
      DOUBLE PRECISION devlpl
      EXTERNAL devlpl
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC dble,log,sqrt
C     ..
C     .. Data statements ..
      DATA xnum/-0.322232431088D0,-1.000000000000D0,-0.342242088547D0,
     +     -0.204231210245D-1,-0.453642210148D-4/
      DATA xden/0.993484626060D-1,0.588581570495D0,0.531103462366D0,
     +     0.103537752850D0,0.38560700634D-2/
C     ..
C     .. Executable Statements ..
      IF (.NOT. (p.LT.1.0E-20)) GO TO 10
      invnor = -10.0
      RETURN

   10 IF (.NOT. (p.GE.1.0)) GO TO 20
      invnor = 10.0
      RETURN

   20 IF (.NOT. (p.LE.0.5D0)) GO TO 30
      sign = -1.0D0
      z = p
      GO TO 40

   30 sign = 1.0D0
      z = 1.0D0 - dble(p)
   40 y = sqrt(-2.0D0*log(z))
      invnor = y + devlpl(xnum,5,y)/devlpl(xden,5,y)
      invnor = sign*invnor
      RETURN

      END
